/* @pjs pauseOnBlur=true; */
//=============================================================================
//  Trajectoire de particules de fluide dans un écoulement stationnaire décrit  
//  par un champ de vitesse eulérien V(x,y).
//  ©J.Roussel -- 1 Dec. 2012

int OX, OY;//Origine du repère
ParticuleFluide[] PF= new ParticuleFluide[20];//On définit 20 particules de fluide 
TriangleFluide TF;
boolean playing = false;

void setup() {
  size(400, 400);
  frameRate(20);
  smooth();
  OX=width/2;//Repère centré au milieu
  OY=height/2;//Repère centré au milieu
  for (int i = 0; i < PF.length; i++) {
    PF[i] = new ParticuleFluide();
  }
  TF=new TriangleFluide();
  background(0);
  text("", 0, 0);  // force Processing to load a font
  String pausetext = " click to play/pause";
  textSize(20);
  float twidth = textWidth(pausetext);
  text(pausetext, (width-twidth)/2, height/2);
}

void draw() {
  if (playing) {
    background(200); 
    for (int i=0;i<20;i++)
      for (int j=0;j<20;j++)
      {
        drawVitesse(20*i, 20*j, 10);
      }
    for (int i=0;i<PF.length;i++) {
      PF[i].velocity=vitesse(PF[i].location.x, PF[i].location.y);
      PF[i].update();
      PF[i].checkEdges();
      PF[i].display();
    }
    TF.velocityA=vitesse(TF.locationA.x, TF.locationA.y);
    TF.velocityB=vitesse(TF.locationB.x, TF.locationB.y);
    TF.velocityC=vitesse(TF.locationC.x, TF.locationC.y);
    TF.update();
    TF.display();
  }
}

//Champ Vecteur vitesse V(x,y) (en pixel par frame)
// ATTENTION ! FAIRE : 
// x=xx-OX
// y=yy-OY
PVector vitesse(float xx, float yy) {
  return new PVector(-0.01*(yy-OY),0.01*(xx-OX));
}

//tracé du vecteur vitesse
void drawVitesse(int posx, int posy, float scaling) {  
  pushMatrix();
  float arrowsize = 4;
  // Translate to location to render vector
  translate(posx, posy);
  stroke(153);
  // Vector heading to get direction (pointing up is a heading of 0)
  rotate(vitesse(posx, posy).heading2D());
  // Scale it to be bigger or smaller if necessary
  float len = vitesse(posx, posy).mag()*scaling;
  // Draw three lines to make an arrow
  line(0, 0, len, 0);
  line(len, 0, len-arrowsize, +arrowsize/2);
  line(len, 0, len-arrowsize, -arrowsize/2);
  popMatrix();
}

/*===== procédures pour déclencher l'anim ou l'arêter après un clic ====*/
void play() {
  playing = true;
  TF=new TriangleFluide();
  for (int i = 0; i < PF.length; i++) {
    PF[i] = new ParticuleFluide();
  }
  loop();
} 
void pause() {
  playing = false;
  noStroke();
  fill(127, 127, 127, 127);
  rect(0, 0, width, height);
  fill(255);
  String pausetext = " click to play/pause";
  textSize(20);
  float twidth = textWidth(pausetext);
  text(pausetext, (width-twidth)/2, height/2);
  textSize(10);
  noLoop();
}
void mouseClicked() {
  if (!playing) { 
    play();
  }
  else { 
    pause();
  }
}

class ParticuleFluide {

  PVector location;
  PVector velocity;

  ParticuleFluide() {
    location = new PVector(random(width), random(height));
    velocity = new PVector(random(-2, 2), random(-2, 2));
  }

  void update() {
    location.add(velocity);
  }

  void display() {
    stroke(0,0,0);
    strokeWeight(4);
//    fill(127);
    point(location.x, location.y);
    strokeWeight(1);
    }

  void checkEdges() {

    if (location.x > width) {
      location.x = 0;
    } 
    else if (location.x < 0) {
      location.x = width;
    }

    if (location.y > height) {
      location.y = 0;
    } 
    else if (location.y < 0) {
      location.y = height;
    }
  }
}

class TriangleFluide {

  PVector locationA;
  PVector locationB;
  PVector locationC;
  PVector velocityA;
  PVector velocityB;
  PVector velocityC;

  TriangleFluide() {
    PVector v1 = new PVector(17,30);
    PVector v2 = new PVector(-17,30);
//    locationA = new PVector(random(width),random(height));
    locationA = new PVector(mouseX,mouseY);
    locationB = PVector.add(locationA,v1);
    locationC = PVector.add(locationA,v2);
    velocityA = new PVector(0,0);
    velocityB = new PVector(0,0);
    velocityC = new PVector(0,0);
  }

  void update() {
    locationA.add(velocityA);
    locationB.add(velocityB);
    locationC.add(velocityC);
  }

  void display() {
    stroke(0);
    fill(175,175,175,100);
    triangle(locationA.x,locationA.y,locationB.x,locationB.y,locationC.x,locationC.y);
    }
}


